#[derive(Debug)]
struct BlogPost {
    id: String,
}

impl BlogPost {
    pub(crate) async fn load_blog_post<P: AsRef<Path>, R: AsRef<Path>>(
        root_dir: R,
        post_dir: P,
    ) -> Result<BlogPost, Box<dyn std::error::Error>> {
        async fn inner(
            root_dir: &Path,
            post_dir: &Path,
        ) -> Result<BlogPost, Box<dyn std::error::Error>> {
            let org_files = {
                let mut ret = Vec::new();
                let org_files_iter = get_org_files(post_dir)?;
                for entry in org_files_iter {
                    ret.push(entry.await??);
                }
                ret
            };
            let parsed_org_files = {
                let mut ret = Vec::new();
                for (path, contents) in org_files.iter() {
                    let parsed = organic::parser::parse_file(contents.as_str(), Some(path))?;
                    ret.push((path, contents, parsed));
                }
                ret
            };

            Ok(BlogPost {
                id: "foo".to_owned(),
            })
        }
        inner(root_dir.as_ref(), post_dir.as_ref()).await
    }
}

fn get_org_files<P: AsRef<Path>>(
    root_dir: P,
) -> Result<impl Iterator<Item = JoinHandle<std::io::Result<(PathBuf, String)>>>, walkdir::Error> {
    let org_files = WalkDir::new(root_dir)
        .into_iter()
        .filter(|e| match e {
            Ok(dir_entry) => {
                dir_entry.file_type().is_file()
                    && Path::new(dir_entry.file_name())
                        .extension()
                        .map(|ext| ext.to_ascii_lowercase() == "org")
                        .unwrap_or(false)
            }
            Err(_) => true,
        })
        .collect::<Result<Vec<_>, _>>()?;
    let org_files = org_files
        .into_iter()
        .map(walkdir::DirEntry::into_path)
        .map(|path| tokio::spawn(read_file(path)));
    Ok(org_files)
}
